# PyFx Pedal Builder: An Open Platform for Developing and Sharing Guitar Effects

## Abstract

Guitar effects algorithms have traditionally been developed using low level languages like C and assembly. While there are performance advantages to this approach, the algorithm development workflow can be slow and convoluted. Using this approach requires recompilation and reflashing of the target hardware anytime there are changes to the guitar effects processing algorithm. The algorithm development process can be significantly sped up using an interpreted language like Python. This is because Python allows for on-the-fly modifications and real-time testing without the need for constant recompilation, enabling a more agile and iterative development process. 

In this talk, I will introduce PyFx Pedal Builder, a novel platform that leverages the power of Python for rapid guitar effects algorithm development. This platform not only simplifies the development workflow, but also opens up new possibilities for creative sound manipulation and experimentation, making it accessible to a broader range of developers, including those who may not have a deep background in low-level programming languages. 


## Description

1) Introduction (30 seconds)
    * Who am I
        * Keaton Scheible
        * Embedded software engineer 
        * Python hobbyist
        * Music enthusiast
1) Overview (30 Seconds)
    * PyFx Pedal Builder
        * What is it?
        * Why was it created?
        * How does it work?
        * Demo creating a guitar effects pedal
        * Where can you learn more?
        * Whats next?
1) What is PyFx Pedal Builder? (3 minutes)
    * Platform for rapid guitar effects algorithm development
    * Provides a GUI for prototyping and controlling guitar effects pedals
        * Add knobs, and footswitches, set the pedal color, etc.
    * Autogenerates template code for implementing the guitar effects algorithm
        * Provides interface to access knob and footswitch settings dynamically at runtime
    * Uses PySide6 as GUI backend and PyAudio as the audio backend
    * Supports audio input from audio files and audio interfaces
1) Why was PyFx Pedal Builder created? (1 minute)
    * I wanted a way to quickly prototype and test guitar effects algorithms
1) How does PyFx Pedal Builder work? (3 minutes)
    * PySide6 GUI for creating pedal prototype
    * PyAudio backend for audio routing
    * Code template generation for easy algorithm development
1) Demo using PyFx Pedal Builder to create a guitar effects pedal (13 Minutes)
    * Prototyping a Guitar Effects Pedal
        * Creating a new pedal
        * Giving the pedal a name
        * Setting the pedal color
        * Adding footswitches to the pedal
        * Naming the footswitches
        * Adding knobs to the pedal
        * Naming the knobs
        * Configuring the footswitches
            * Setting the footswitch type (Latching, Momentary or Mode)
            * Enabling footswitch display
        * Configuring the knobs
            * Setting the knob response type (Linear or Logarithmic)
            * Setting the knob range, default value and precision
            * Enabling knob display
        * Saving the pedal
    * Designing the algorithm
        * A look at the autogenerated guitar effect algorithm template 
        * How to access the knob and footswitch values in code
        * Implementing a guitar effects algorithm
        * Testing the guitar effects algorithm
    * Creating pedal variants
        * Creating a new pedal variant
        * Selecting the pedal variant
    * Pedal Modifications
        * Changing knob, footswitch or pedal name automatically updates the guitar effects algorithm code
    * Demo of other guitar effects algorithms built using PyFx Pedal Builder
1) Where you can learn more about PyFx Pedal Builder (2 Minute)
    * PyPi
    * Github
    * pyfx.dev (not yet active, but will be by the time of the conference)
1) Whats Next? (2 Minute)
    * Listen to feedback. I am open to suggestion for improvements.
    * Build a community to share guitar effects algorithms
    * Looking into hardware solutions that can run algorithms designed with PyFx Pedal Builder

## Notes

* Github: https://github.com/thatdspguy/pyfx/tree/dev
* PyPi: https://pypi.org/project/pyfx-dev/

I am actively developing this project and there is not much documentation about the project online yet, but there will be by the time of the conference. If you would like to see a demo of PyFx Pedal Builder in action, please reach out to me at thatdspguy@gmail.com or call me at (503) 888-7664.

For the presentation I will have a powerpoint slideshow and would like to share my computer screen with the audience for the demo of PyFx Pedal Builder. I would also like to connect my computer to the venue audio system if possible so I can demo the guitar effects algorithms in action. If this is not possible, I am able to bring a speaker that can be connected to my computers audio system to demo the guitar effects if that is ok. 

I am really excited about this project and am looking forward to sharing it with the Python community and getting some feedback. Please let me know if you have any questions at all!

Thank you, 
Keaton Scheible

